@@ -2,7 +2,7 @@
 // detail/win_iocp_io_service.hpp
 // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2013 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2016 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -207,6 +207,9 @@ private:
   // either 0 or 1).
   BOOST_ASIO_DECL size_t do_one(bool block, boost::system::error_code& ec);
 
+  // Helper to calculate the GetQueuedCompletionStatus timeout.
+  BOOST_ASIO_DECL static DWORD get_gqcs_timeout();
+
   // Helper function to add a new timer queue.
   BOOST_ASIO_DECL void do_add_timer_queue(timer_queue_base& queue);
 
@@ -246,11 +249,11 @@ private:
 
   enum
   {
-    // Timeout to use with GetQueuedCompletionStatus. Some versions of windows
-    // have a "bug" where a call to GetQueuedCompletionStatus can appear stuck
-    // even though there are events waiting on the queue. Using a timeout helps
-    // to work around the issue.
-    gqcs_timeout = 500,
+    // Timeout to use with GetQueuedCompletionStatus on older versions of
+    // Windows. Some versions of windows have a "bug" where a call to
+    // GetQueuedCompletionStatus can appear stuck even though there are events
+    // waiting on the queue. Using a timeout helps to work around the issue.
+    default_gqcs_timeout = 500,
 
     // Maximum waitable timer timeout, in milliseconds.
     max_timeout_msec = 5 * 60 * 1000,
@@ -268,6 +271,9 @@ private:
     overlapped_contains_result = 2
   };
 
+  // Timeout to use with GetQueuedCompletionStatus.
+  const DWORD gqcs_timeout_;
+
   // Function object for processing timeouts in a background thread.
   struct timer_thread_function;
   friend struct timer_thread_function;
