@@ -21,6 +21,7 @@
 
 #include <boost/container/detail/config_begin.hpp>
 #include <boost/container/detail/workaround.hpp>
+#include <boost/move/utility_core.hpp>
 
 #include <boost/container/detail/type_traits.hpp>
 #include <cstddef>   //std::size_t
@@ -42,13 +43,13 @@ class tuple<Head, Tail...>
    typedef tuple<Tail...> inherited;
 
    public:
-   tuple() { }
+   tuple()
+      : inherited(), m_head()
+   {}
 
-   // implicit copy-constructor is okay
-   // Construct tuple from separate arguments.
-   tuple(typename add_const_reference<Head>::type v,
-         typename add_const_reference<Tail>::type... vtail)
-   : inherited(vtail...), m_head(v)
+   template<class U, class ...Args>
+   tuple(U &&u, Args && ...args)
+      : inherited(::boost::forward<Args>(args)...), m_head(::boost::forward<U>(u))
    {}
 
    // Construct tuple from another tuple.
@@ -77,8 +78,8 @@ class tuple<Head, Tail...>
 
 
 template<typename... Values>
-tuple<Values&&...> tie_forward(Values&&... values)
-{ return tuple<Values&&...>(values...); }
+tuple<Values&&...> forward_as_tuple(Values&&... values)
+{ return tuple<Values&&...>(::boost::forward<Values>(values)...); }
 
 template<int I, typename Tuple>
 struct tuple_element;
@@ -135,18 +136,18 @@ typename get_impl<I, tuple<Values...> >::const_type get(const tuple<Values...>&
 // in a function call.
 ////////////////////////////////////////////////////
 
-template<int... Indexes>
+template<std::size_t ... Indexes>
 struct index_tuple{};
 
 template<std::size_t Num, typename Tuple = index_tuple<> >
 struct build_number_seq;
 
-template<std::size_t Num, int... Indexes>
+template<std::size_t Num, std::size_t ... Indexes>
 struct build_number_seq<Num, index_tuple<Indexes...> >
    : build_number_seq<Num - 1, index_tuple<Indexes..., sizeof...(Indexes)> >
 {};
 
-template<int... Indexes>
+template<std::size_t ... Indexes>
 struct build_number_seq<0, index_tuple<Indexes...> >
 {  typedef index_tuple<Indexes...> type;  };
 
