@@ -21,6 +21,8 @@
 // C++11 interface
 ///////////////////////////////////////////////////////////////////////////////
 #include <boost/fusion/support/sequence_base.hpp>
+#include <boost/fusion/support/void.hpp>
+#include <boost/fusion/support/detail/enabler.hpp>
 #include <boost/fusion/support/detail/access.hpp>
 #include <boost/fusion/support/is_sequence.hpp>
 #include <boost/fusion/container/deque/detail/keyed_element.hpp>
@@ -50,18 +52,20 @@ namespace boost { namespace fusion
         typedef bidirectional_traversal_tag category;
         typedef mpl::int_<0> size;
         typedef mpl::int_<0> next_up;
-        typedef mpl::int_<0> next_down;
+        typedef mpl::int_<-1> next_down;
         typedef mpl::false_ is_view;
 
         template <typename Sequence>
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque(Sequence const&,
             typename enable_if<
                 mpl::and_<
                     traits::is_sequence<Sequence>
-                  , result_of::empty<Sequence>>>::type* /*dummy*/ = 0)
+                  , result_of::empty<Sequence>>, detail::enabler_>::type = detail::enabler) BOOST_NOEXCEPT
         {}
 
-        deque() {}
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        deque() BOOST_NOEXCEPT {}
     };
 
     template <typename Head, typename ...Tail>
@@ -74,63 +78,86 @@ namespace boost { namespace fusion
         typedef typename detail::deque_keyed_values<Head, Tail...>::type base;
         typedef mpl::int_<(sizeof ...(Tail) + 1)> size;
         typedef mpl::int_<size::value> next_up;
-        typedef mpl::int_<((size::value == 0) ? 0 : -1)> next_down;
+        typedef mpl::int_<-1> next_down;
         typedef mpl::false_ is_view;
 
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque()
         {}
 
-        template <typename ...Elements>
-        deque(deque<Elements...> const& seq)
+        template <typename Head_, typename ...Tail_, typename =
+            typename enable_if<is_convertible<Head_, Head> >::type
+        >
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        deque(deque<Head_, Tail_...> const& seq)
           : base(seq)
         {}
 
-        template <typename ...Elements>
-        deque(deque<Elements...>& seq)
+        template <typename Head_, typename ...Tail_, typename =
+            typename enable_if<is_convertible<Head_, Head> >::type
+        >
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        deque(deque<Head_, Tail_...>& seq)
           : base(seq)
         {}
 
 #if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
-        template <typename ...Elements>
-        deque(deque<Elements...>&& seq)
-          : base(std::forward<deque<Elements...>>(seq))
+        template <typename Head_, typename ...Tail_, typename =
+            typename enable_if<is_convertible<Head_, Head> >::type
+        >
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        deque(deque<Head_, Tail_...>&& seq)
+          : base(std::forward<deque<Head_, Tail_...>>(seq))
         {}
 #endif
 
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque(deque const& seq)
           : base(seq)
         {}
 
 #if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque(deque&& seq)
           : base(std::forward<deque>(seq))
         {}
 #endif
 
-        explicit deque(Head const& head, Tail const&... tail)
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        explicit deque(typename detail::call_param<Head>::type head
+                     , typename detail::call_param<Tail>::type... tail)
           : base(detail::deque_keyed_values<Head, Tail...>::construct(head, tail...))
         {}
 
-        template <typename Head_, typename ...Tail_>
-        explicit deque(Head_ const& head, Tail_ const&... tail)
-          : base(detail::deque_keyed_values<Head_, Tail_...>::construct(head, tail...))
-        {}
-
 #if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
-        template <typename Head_, typename ...Tail_>
+        template <typename Head_, typename ...Tail_, typename =
+            typename enable_if<is_convertible<Head_, Head> >::type
+        >
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         explicit deque(Head_&& head, Tail_&&... tail)
           : base(detail::deque_keyed_values<Head, Tail...>
-                ::forward_(std::forward<Head_>(head), std::forward<Tail_>(tail)...))
+                ::forward_(BOOST_FUSION_FWD_ELEM(Head_, head), BOOST_FUSION_FWD_ELEM(Tail_, tail)...))
+        {}
+#else
+        template <typename Head_, typename ...Tail_, typename =
+            typename enable_if<is_convertible<Head_, Head> >::type
+        >
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+        explicit deque(Head_ const& head, Tail_ const&... tail)
+          : base(detail::deque_keyed_values<Head_, Tail_...>::construct(head, tail...))
         {}
 #endif
 
         template <typename Sequence>
+        BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         explicit deque(Sequence const& seq
-          , typename disable_if<is_convertible<Sequence, Head> >::type* /*dummy*/ = 0)
+          , typename disable_if<is_convertible<Sequence, Head>, detail::enabler_>::type = detail::enabler
+          , typename enable_if<traits::is_sequence<Sequence>, detail::enabler_>::type = detail::enabler)
           : base(base::from_iterator(fusion::begin(seq)))
         {}
 
         template <typename ...Elements>
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque& operator=(deque<Elements...> const& rhs)
         {
             base::operator=(rhs);
@@ -138,6 +165,7 @@ namespace boost { namespace fusion
         }
 
         template <typename T>
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque& operator=(T const& rhs)
         {
             base::operator=(rhs);
@@ -146,9 +174,10 @@ namespace boost { namespace fusion
 
 #if !defined(BOOST_NO_CXX11_RVALUE_REFERENCES)
         template <typename T>
+        BOOST_CXX14_CONSTEXPR BOOST_FUSION_GPU_ENABLED
         deque& operator=(T&& rhs)
         {
-            base::operator=(std::forward<T>(rhs));
+            base::operator=(BOOST_FUSION_FWD_ELEM(T, rhs));
             return *this;
         }
 #endif
