@@ -1,5 +1,5 @@
 /*=============================================================================
-    Copyright (c) 2001-2006 Joel de Guzman
+    Copyright (c) 2001-2011 Joel de Guzman
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying 
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -7,6 +7,7 @@
 #if !defined(FUSION_SEQUENCE_VIEW_10022005_0620)
 #define FUSION_SEQUENCE_VIEW_10022005_0620
 
+#include <boost/fusion/support/config.hpp>
 #include <boost/fusion/view/filter_view.hpp>
 #include <boost/fusion/view/iterator_range.hpp>
 #include <boost/fusion/view/joint_view.hpp>
@@ -15,5 +16,6 @@
 #include <boost/fusion/view/reverse_view.hpp>
 #include <boost/fusion/view/transform_view.hpp>
 #include <boost/fusion/view/zip_view.hpp>
+#include <boost/fusion/view/flatten_view.hpp>
 
 #endif
