@@ -11,7 +11,11 @@
 #ifndef BOOST_INTERPROCESS_LOCK_OPTIONS_HPP
 #define BOOST_INTERPROCESS_LOCK_OPTIONS_HPP
 
-#if (defined _MSC_VER) && (_MSC_VER >= 1200)
+#ifndef BOOST_CONFIG_HPP
+#  include <boost/config.hpp>
+#endif
+#
+#if defined(BOOST_HAS_PRAGMA_ONCE)
 #  pragma once
 #endif
 
@@ -23,9 +27,13 @@
 
 namespace boost {
 
+#if !defined(BOOST_INTERPROCESS_DOXYGEN_INVOKED)
+
 namespace posix_time
 {  class ptime;   }
 
+#endif   //#if !defined(BOOST_INTERPROCESS_DOXYGEN_INVOKED)
+
 namespace interprocess {
 
 //!Type to indicate to a mutex lock constructor that must not lock the mutex.
