@@ -8,18 +8,14 @@
 #ifndef BOOST_IOSTREAMS_DETAIL_DEFAULT_ARG_HPP_INCLUDED
 #define BOOST_IOSTREAMS_DETAIL_DEFAULT_ARG_HPP_INCLUDED
 
-#if defined(_MSC_VER) && (_MSC_VER >= 1020)
+#if defined(_MSC_VER)
 # pragma once
 #endif            
 
 #include <boost/config.hpp>
 #include <boost/detail/workaround.hpp>
 
-#if BOOST_WORKAROUND(BOOST_MSVC, < 1300)
-# include <boost/mpl/identity.hpp>
-# define BOOST_IOSTREAMS_DEFAULT_ARG(arg) mpl::identity< arg >::type
-#else
-# define BOOST_IOSTREAMS_DEFAULT_ARG(arg) arg
-#endif
+// Obsolete. Remove.
+#define BOOST_IOSTREAMS_DEFAULT_ARG(arg) arg
 
 #endif // #ifndef BOOST_IOSTREAMS_DETAIL_DEFAULT_ARG_HPP_INCLUDED
