@@ -21,6 +21,7 @@
 #include <boost/range/iterator_range.hpp>
 #include <boost/range/begin.hpp>
 #include <boost/range/end.hpp>
+#include <boost/range/concepts.hpp>
 #include <boost/iterator/iterator_adaptor.hpp>
 #include <boost/next_prior.hpp>
 
@@ -62,54 +63,44 @@ namespace boost
                 , pred_t(pred)
                 , m_last(last)
             {
-                move_to_next_valid();
             }
 
             template<class OtherIter>
             skip_iterator( const skip_iterator<OtherIter, pred_t, default_pass>& other )
             : base_t(other.base())
             , pred_t(other)
-            , m_last(other.m_last) {}
-
-            void move_to_next_valid()
+            , m_last(other.m_last)
             {
-                iter_t& it = this->base_reference();
-                pred_t& bi_pred = *this;
-                if (it != m_last)
-                {
-                    if (default_pass)
-                    {
-                        iter_t nxt = ::boost::next(it);
-                        while (nxt != m_last && !bi_pred(*it, *nxt))
-                        {
-                            ++it;
-                            ++nxt;
-                        }
-                    }
-                    else
-                    {
-                        iter_t nxt = ::boost::next(it);
-                        for(; nxt != m_last; ++it, ++nxt)
-                        {
-                            if (bi_pred(*it, *nxt))
-                            {
-                                break;
-                            }
-                        }
-                        if (nxt == m_last)
-                        {
-                            it = m_last;
-                        }
-                    }
-                }
             }
 
             void increment()
             {
                 iter_t& it = this->base_reference();
                 BOOST_ASSERT( it != m_last );
+                pred_t& bi_pred = *this;
+                iter_t prev = it;
                 ++it;
-                move_to_next_valid();
+                if (it != m_last)
+                {
+                    if (default_pass)
+                    {
+                        while (it != m_last && !bi_pred(*prev, *it))
+                        {
+                            ++it;
+                            ++prev;
+                        }
+                    }
+                    else
+                    {
+                        for (; it != m_last; ++it, ++prev)
+                        {
+                            if (bi_pred(*prev, *it))
+                            {
+                                break;
+                            }
+                        }
+                    }
+                }
             }
 
             iter_t m_last;
@@ -143,10 +134,6 @@ namespace boost
                          skip_iter(boost::end(r), boost::end(r), p))
             {
             }
-
-        private:
-            P m_pred;
-            R* m_range;
         };
 
         template< class T >
@@ -168,6 +155,8 @@ namespace boost
         operator|( ForwardRng& r,
                    const adjacent_holder<BinPredicate>& f )
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<ForwardRng>));
+
             return adjacent_filtered_range<BinPredicate, ForwardRng, true>( f.val, r );
         }
 
@@ -176,6 +165,8 @@ namespace boost
         operator|( const ForwardRng& r,
                    const adjacent_holder<BinPredicate>& f )
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<const ForwardRng>));
+
             return adjacent_filtered_range<BinPredicate,
                                            const ForwardRng, true>( f.val, r );
         }
@@ -185,6 +176,7 @@ namespace boost
         operator|( ForwardRng& r,
                    const adjacent_excl_holder<BinPredicate>& f )
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<ForwardRng>));
             return adjacent_filtered_range<BinPredicate, ForwardRng, false>( f.val, r );
         }
 
@@ -193,6 +185,7 @@ namespace boost
         operator|( const ForwardRng& r,
                    const adjacent_excl_holder<BinPredicate>& f )
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<const ForwardRng>));
             return adjacent_filtered_range<BinPredicate,
                                            const ForwardRng, false>( f.val, r );
         }
@@ -221,6 +214,7 @@ namespace boost
         inline adjacent_filtered_range<BinPredicate, ForwardRng, true>
         adjacent_filter(ForwardRng& rng, BinPredicate filter_pred)
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<ForwardRng>));
             return adjacent_filtered_range<BinPredicate, ForwardRng, true>(filter_pred, rng);
         }
 
@@ -228,6 +222,7 @@ namespace boost
         inline adjacent_filtered_range<BinPredicate, const ForwardRng, true>
         adjacent_filter(const ForwardRng& rng, BinPredicate filter_pred)
         {
+            BOOST_RANGE_CONCEPT_ASSERT((ForwardRangeConcept<const ForwardRng>));
             return adjacent_filtered_range<BinPredicate, const ForwardRng, true>(filter_pred, rng);
         }
 
