@@ -1,6 +1,6 @@
 /*=============================================================================
-    Copyright (c) 2001-2010 Hartmut Kaiser
-    Copyright (c) 2001-2010 Joel de Guzman
+    Copyright (c) 2001-2011 Hartmut Kaiser
+    Copyright (c) 2001-2011 Joel de Guzman
 
     Distributed under the Boost Software License, Version 1.0. (See accompanying
     file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -167,6 +167,14 @@ namespace boost { namespace spirit { namespace char_encoding
         /*  }  125  7d */   BOOST_CC_PUNCT,
         /*  ~  126  7e */   BOOST_CC_PUNCT,
         /* DEL 127  7f */   BOOST_CC_CTRL,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
     };
 
     ///////////////////////////////////////////////////////////////////////////
@@ -188,86 +196,86 @@ namespace boost { namespace spirit { namespace char_encoding
             return isascii_(ch);
         }
 
-        static int
+        static bool
         isalnum(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
             return (ascii_char_types[ch] & BOOST_CC_ALPHA)
                 || (ascii_char_types[ch] & BOOST_CC_DIGIT);
         }
 
-        static int
+        static bool
         isalpha(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_ALPHA);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_ALPHA) ? true : false;
         }
 
-        static int
+        static bool
         isdigit(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_DIGIT);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_DIGIT) ? true : false;
         }
 
-        static int
+        static bool
         isxdigit(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_XDIGIT);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_XDIGIT) ? true : false;
         }
 
-        static int
+        static bool
         iscntrl(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_CTRL);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_CTRL) ? true : false;
         }
 
-        static int
+        static bool
         isgraph(int ch)
         {
             return ('\x21' <= ch && ch <= '\x7e');
         }
 
-        static int
+        static bool
         islower(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_LOWER);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_LOWER) ? true : false;
         }
 
-        static int
+        static bool
         isprint(int ch)
         {
             return ('\x20' <= ch && ch <= '\x7e');
         }
 
-        static int
+        static bool
         ispunct(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_PUNCT);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_PUNCT) ? true : false;
         }
 
-        static int
+        static bool
         isspace(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_SPACE);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_SPACE) ? true : false;
         }
 
-        static int
+        static bool
         isblank BOOST_PREVENT_MACRO_SUBSTITUTION (int ch)
         {
             return ('\x09' == ch || '\x20' == ch);
         }
 
-        static int
+        static bool
         isupper(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
-            return (ascii_char_types[ch] & BOOST_CC_UPPER);
+            BOOST_ASSERT(0 == (ch & ~UCHAR_MAX));
+            return (ascii_char_types[ch] & BOOST_CC_UPPER) ? true : false;
         }
 
         ///////////////////////////////////////////////////////////////////////
@@ -277,14 +285,12 @@ namespace boost { namespace spirit { namespace char_encoding
         static int
         tolower(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
             return isupper(ch) ? (ch - 'A' + 'a') : ch;
         }
 
         static int
         toupper(int ch)
         {
-            BOOST_ASSERT(isascii_(ch));
             return islower(ch) ? (ch - 'a' + 'A') : ch;
         }
 
@@ -310,4 +316,3 @@ namespace boost { namespace spirit { namespace char_encoding
 #undef BOOST_CC_SPACE
 
 #endif
-
