@@ -1,15 +1,14 @@
 /*
- Copyright (c) 2014 Glen Joseph Fernandes
- glenfe at live dot com
+(c) 2014 Glen Joseph Fernandes
+<glenjofe -at- gmail.com>
 
- Distributed under the Boost Software License,
- Version 1.0. (See accompanying file LICENSE_1_0.txt
- or copy at http://boost.org/LICENSE_1_0.txt)
+Distributed under the Boost Software
+License, Version 1.0.
+http://boost.org/LICENSE_1_0.txt
 */
 #include <boost/align/aligned_alloc.hpp>
 #include <boost/align/is_aligned.hpp>
 #include <boost/core/lightweight_test.hpp>
-#include <cstddef>
 #include <cstring>
 
 void test(std::size_t alignment)
@@ -18,7 +17,7 @@ void test(std::size_t alignment)
         void* p = boost::alignment::aligned_alloc(alignment,
             alignment);
         BOOST_TEST(p != 0);
-        BOOST_TEST(boost::alignment::is_aligned(alignment, p));
+        BOOST_TEST(boost::alignment::is_aligned(p, alignment));
         std::memset(p, 0, 1);
         boost::alignment::aligned_free(p);
     }
@@ -26,7 +25,7 @@ void test(std::size_t alignment)
         void* p = boost::alignment::aligned_alloc(alignment,
             alignment + 1);
         BOOST_TEST(p != 0);
-        BOOST_TEST(boost::alignment::is_aligned(alignment, p));
+        BOOST_TEST(boost::alignment::is_aligned(p, alignment));
         std::memset(p, 0, 1);
         boost::alignment::aligned_free(p);
     }
@@ -34,11 +33,10 @@ void test(std::size_t alignment)
         void* p = boost::alignment::aligned_alloc(alignment,
             alignment - 1);
         BOOST_TEST(p != 0);
-        BOOST_TEST(boost::alignment::is_aligned(alignment, p));
+        BOOST_TEST(boost::alignment::is_aligned(p, alignment));
         std::memset(p, 0, 1);
         boost::alignment::aligned_free(p);
-    }
-    {
+    } else {
         void* p = boost::alignment::aligned_alloc(alignment, 0);
         boost::alignment::aligned_free(p);
     }
@@ -46,9 +44,14 @@ void test(std::size_t alignment)
 
 int main()
 {
-    for (std::size_t i = 1; i <= 128; i <<= 1) {
-        test(i);
-    }
+    test(1);
+    test(2);
+    test(4);
+    test(8);
+    test(16);
+    test(32);
+    test(64);
+    test(128);
 
     return boost::report_errors();
 }
