@@ -11,7 +11,7 @@
 
 //  See http://www.boost.org/libs/config for most recent version.
 //
-//  Revision $Id: config_info.cpp 85088 2013-07-20 17:17:10Z andysem $
+//  Revision $Id$
 //
 
 #include <boost/config.hpp>
@@ -192,6 +192,7 @@ void print_compiler_macros()
   PRINT_MACRO(__hppa);
   PRINT_MACRO(__CYGWIN__);
   PRINT_MACRO(__MINGW32__);
+  PRINT_MACRO(__GXX_RTTI);
   // HP aCC:
   PRINT_MACRO(__HP_aCC);
   PRINT_MACRO(_HPACC_);
@@ -293,6 +294,8 @@ void print_compiler_macros()
   PRINT_MACRO(_OPENMPT);
   PRINT_MACRO(_PGO_INSTRUMENT);
   PRINT_MACRO(__QMSPP_); 
+  PRINT_MACRO(__INTEL_RTTI__);
+  PRINT_MACRO(__INTEL_CXX11_MODE__);
 
   // Cray options:
   PRINT_MACRO(_CRAYC); 
@@ -327,6 +330,13 @@ void print_compiler_macros()
   PRINT_MACRO(_NTO_VERSION);
   PRINT_MACRO(__OPEN64__);
   PRINT_MACRO(__open64);
+  PRINT_MACRO(__NVCC__);
+  PRINT_MACRO(__CUDACC__);
+  PRINT_MACRO(__CUDACC_RDC__);
+  PRINT_MACRO(__CUDACC_VER_MAJOR__);
+  PRINT_MACRO(__CUDACC_VER_MINOR__);
+  PRINT_MACRO(__CUDACC_VER_BUILD__);
+  PRINT_MACRO(__CUDACC_VER__);
 }
 
 void print_stdlib_macros()
@@ -934,6 +944,7 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_HAS_CLOCK_GETTIME);
    PRINT_MACRO(BOOST_HAS_DIRENT_H);
    PRINT_MACRO(BOOST_HAS_EXPM1);
+   PRINT_MACRO(BOOST_HAS_FLOAT128);
    PRINT_MACRO(BOOST_HAS_FTIME);
    PRINT_MACRO(BOOST_HAS_GETSYSTEMTIMEASFILETIME);
    PRINT_MACRO(BOOST_HAS_GETTIMEOFDAY);
@@ -992,6 +1003,7 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CV_VOID_SPECIALIZATIONS);
    PRINT_MACRO(BOOST_NO_CWCHAR);
    PRINT_MACRO(BOOST_NO_CWCTYPE);
+   PRINT_MACRO(BOOST_NO_CXX11_ADDRESSOF);
    PRINT_MACRO(BOOST_NO_CXX11_ALIGNAS);
    PRINT_MACRO(BOOST_NO_CXX11_ALLOCATOR);
    PRINT_MACRO(BOOST_NO_CXX11_ATOMIC_SMART_PTR);
@@ -1006,8 +1018,11 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CXX11_DELETED_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_CXX11_EXPLICIT_CONVERSION_OPERATORS);
    PRINT_MACRO(BOOST_NO_CXX11_EXTERN_TEMPLATE);
+   PRINT_MACRO(BOOST_NO_CXX11_FINAL);
+   PRINT_MACRO(BOOST_NO_CXX11_FIXED_LENGTH_VARIADIC_TEMPLATE_EXPANSION_PACKS);
    PRINT_MACRO(BOOST_NO_CXX11_FUNCTION_TEMPLATE_DEFAULT_ARGS);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_ARRAY);
+   PRINT_MACRO(BOOST_NO_CXX11_HDR_ATOMIC);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_CHRONO);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_CODECVT);
    PRINT_MACRO(BOOST_NO_CXX11_HDR_CONDITION_VARIABLE);
@@ -1030,21 +1045,35 @@ void print_boost_macros()
    PRINT_MACRO(BOOST_NO_CXX11_LAMBDAS);
    PRINT_MACRO(BOOST_NO_CXX11_LOCAL_CLASS_TEMPLATE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_CXX11_NOEXCEPT);
+   PRINT_MACRO(BOOST_NO_CXX11_NON_PUBLIC_DEFAULTED_FUNCTIONS);
    PRINT_MACRO(BOOST_NO_CXX11_NULLPTR);
    PRINT_MACRO(BOOST_NO_CXX11_NUMERIC_LIMITS);
    PRINT_MACRO(BOOST_NO_CXX11_RANGE_BASED_FOR);
    PRINT_MACRO(BOOST_NO_CXX11_RAW_LITERALS);
+   PRINT_MACRO(BOOST_NO_CXX11_REF_QUALIFIERS);
    PRINT_MACRO(BOOST_NO_CXX11_RVALUE_REFERENCES);
    PRINT_MACRO(BOOST_NO_CXX11_SCOPED_ENUMS);
    PRINT_MACRO(BOOST_NO_CXX11_SMART_PTR);
    PRINT_MACRO(BOOST_NO_CXX11_STATIC_ASSERT);
+   PRINT_MACRO(BOOST_NO_CXX11_STD_ALIGN);
    PRINT_MACRO(BOOST_NO_CXX11_TEMPLATE_ALIASES);
+   PRINT_MACRO(BOOST_NO_CXX11_THREAD_LOCAL);
    PRINT_MACRO(BOOST_NO_CXX11_TRAILING_RESULT_TYPES);
    PRINT_MACRO(BOOST_NO_CXX11_UNICODE_LITERALS);
    PRINT_MACRO(BOOST_NO_CXX11_UNIFIED_INITIALIZATION_SYNTAX);
    PRINT_MACRO(BOOST_NO_CXX11_USER_DEFINED_LITERALS);
    PRINT_MACRO(BOOST_NO_CXX11_VARIADIC_MACROS);
    PRINT_MACRO(BOOST_NO_CXX11_VARIADIC_TEMPLATES);
+   PRINT_MACRO(BOOST_NO_CXX14_AGGREGATE_NSDMI);
+   PRINT_MACRO(BOOST_NO_CXX14_BINARY_LITERALS);
+   PRINT_MACRO(BOOST_NO_CXX14_CONSTEXPR);
+   PRINT_MACRO(BOOST_NO_CXX14_DECLTYPE_AUTO);
+   PRINT_MACRO(BOOST_NO_CXX14_DIGIT_SEPARATORS);
+   PRINT_MACRO(BOOST_NO_CXX14_GENERIC_LAMBDAS);
+   PRINT_MACRO(BOOST_NO_CXX14_HDR_SHARED_MUTEX);
+   PRINT_MACRO(BOOST_NO_CXX14_INITIALIZED_LAMBDA_CAPTURES);
+   PRINT_MACRO(BOOST_NO_CXX14_RETURN_TYPE_DEDUCTION);
+   PRINT_MACRO(BOOST_NO_CXX14_VARIABLE_TEMPLATES);
    PRINT_MACRO(BOOST_NO_DEPENDENT_NESTED_DERIVATIONS);
    PRINT_MACRO(BOOST_NO_DEPENDENT_TYPES_IN_TEMPLATE_VALUE_PARAMETERS);
    PRINT_MACRO(BOOST_NO_EXCEPTIONS);
@@ -1106,6 +1135,12 @@ void print_boost_macros()
 
 
 
+
+
+
+
+
+
    // END GENERATED BLOCK
 
    PRINT_MACRO(BOOST_INTEL);
