@@ -12,7 +12,7 @@ using boost::convert;
 //'boost::cnv::by_default' : class has no constructors.
 
 int
-main(int argc, char const* argv[])
+main(int, char const* [])
 {
     int    i = convert<int>("123").value();
     string s = convert<string>(123).value();
