@@ -1,7 +1,7 @@
 // Boost.Geometry Index
 // Additional tests
 
-// Copyright (c) 2011-2013 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2011-2015 Adam Wulkiewicz, Lodz, Poland.
 
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
@@ -14,11 +14,16 @@
 #include <boost/chrono.hpp>
 #include <boost/foreach.hpp>
 #include <boost/random.hpp>
+#include <boost/range/algorithm/copy.hpp>
 
+#include <boost/geometry.hpp>
 #include <boost/geometry/index/rtree.hpp>
 #include <boost/geometry/geometries/linestring.hpp>
 #include <boost/geometry/geometries/segment.hpp>
 
+#include <boost/geometry/index/detail/rtree/utilities/are_levels_ok.hpp>
+#include <boost/geometry/index/detail/rtree/utilities/are_boxes_ok.hpp>
+
 namespace bg = boost::geometry;
 namespace bgi = bg::index;
 
@@ -26,8 +31,10 @@ typedef bg::model::point<double, 2, bg::cs::cartesian> P;
 typedef bg::model::box<P> B;
 typedef bg::model::linestring<P> LS;
 typedef bg::model::segment<P> S;
-typedef B V;
 //typedef P V;
+typedef B V;
+//typedef S V;
+//#define SEGMENT_INDEXABLE
 
 template <typename V>
 struct generate_value {};
@@ -38,6 +45,12 @@ struct generate_value<B>
     static inline B apply(float x, float y) { return B(P(x - 0.5f, y - 0.5f), P(x + 0.5f, y + 0.5f)); }
 };
 
+template <>
+struct generate_value<S>
+{
+    static inline S apply(float x, float y) { return S(P(x - 0.5f, y - 0.5f), P(x + 0.5f, y + 0.5f)); }
+};
+
 template <>
 struct generate_value<P>
 {
@@ -129,7 +142,10 @@ int main()
             RT t(values.begin(), values.end());
 
             dur_t time = clock_t::now() - start;
-            std::cout << time << " - pack " << values_count << '\n';
+            std::cout << time << " - pack " << values_count /*<< '\n'*/;
+
+            std::cout << (bgi::detail::rtree::utilities::are_levels_ok(t) ? " ok" : " NOK")
+                      << (bgi::detail::rtree::utilities::are_boxes_ok(t) ? " ok\n" : "NOK\n");
 
             {
                 clock_t::time_point start = clock_t::now();
@@ -154,7 +170,10 @@ int main()
             clock_t::time_point start = clock_t::now();
             t.insert(values);
             dur_t time = clock_t::now() - start;
-            std::cout << time << " - insert " << values_count << '\n';
+            std::cout << time << " - insert " << values_count /*<< '\n'*/;
+
+            std::cout << (bgi::detail::rtree::utilities::are_levels_ok(t) ? " ok" : " NOK")
+                      << (bgi::detail::rtree::utilities::are_boxes_ok(t) ? " ok\n" : "NOK\n");
         }
 
         
@@ -278,6 +297,7 @@ int main()
             std::cout << time << " - range type-erased qbegin(B) qend() " << queries_count << " found " << temp << '\n';
         }
 
+#ifndef SEGMENT_INDEXABLE
         {
             clock_t::time_point start = clock_t::now();
             size_t temp = 0;
@@ -304,6 +324,7 @@ int main()
             dur_t time = clock_t::now() - start;
             std::cout << time << " - query(i && !w && !c) " << queries_count << " found " << temp << '\n';
         }
+#endif
 
         result.clear();
 
@@ -376,6 +397,7 @@ int main()
         }
 
 #ifdef BOOST_GEOMETRY_INDEX_DETAIL_EXPERIMENTAL
+#ifndef SEGMENT_INDEXABLE
 
         {
             LS ls;
@@ -436,6 +458,7 @@ int main()
             dur_t time = clock_t::now() - start;
             std::cout << time << " - query(path(S, " << path_values_count << ")) " << path_queries_count2 << " found " << temp << '\n';
         }
+#endif
 #endif
         {
             clock_t::time_point start = clock_t::now();
@@ -448,6 +471,8 @@ int main()
             }
             dur_t time = clock_t::now() - start;
             std::cout << time << " - remove " << values_count / 10 << '\n';
+
+            std::cout << (bgi::detail::rtree::utilities::are_boxes_ok(t) ? " boxes ok\n" : "boxes NOT ok\n");
         }
 
         std::cout << "------------------------------------------------\n";
