@@ -1,8 +1,8 @@
 // Boost.Geometry (aka GGL, Generic Geometry Library)
 
-// Copyright (c) 2007-2012 Barend Gehrels, Amsterdam, the Netherlands.
-// Copyright (c) 2008-2012 Bruno Lalande, Paris, France.
-// Copyright (c) 2009-2012 Mateusz Loskot, London, UK.
+// Copyright (c) 2007-2015 Barend Gehrels, Amsterdam, the Netherlands.
+// Copyright (c) 2008-2015 Bruno Lalande, Paris, France.
+// Copyright (c) 2009-2015 Mateusz Loskot, London, UK.
 
 // Parts of Boost.Geometry are redesigned from Geodan's Geographic Library
 // (geolib/GGL), copyright (c) 1995-2010 Geodan, Amsterdam, the Netherlands.
@@ -15,16 +15,40 @@
 #ifndef GEOMETRY_TEST_GEOMETRY_TEST_COMMON_HPP
 #define GEOMETRY_TEST_GEOMETRY_TEST_COMMON_HPP
 
-#if defined(_MSC_VER)
+#include <boost/config.hpp>
+
+// Determine debug/release mode
+// (it would be convenient if Boost.Config or Boost.Test would define this)
+// Note that they might be combined (e.g. for optimize+no inline)
+#if defined (BOOST_CLANG) || defined(BOOST_GCC)
+#if defined(__OPTIMIZE__)
+    #define BOOST_GEOMETRY_COMPILER_MODE_RELEASE
+#endif
+#if defined(__NO_INLINE__)
+#define BOOST_GEOMETRY_COMPILER_MODE_DEBUG
+#endif
+#endif
+
+#if defined(BOOST_MSVC)
+#if defined(_DEBUG)
+#define BOOST_GEOMETRY_COMPILER_MODE_DEBUG
+#else
+#define BOOST_GEOMETRY_COMPILER_MODE_RELEASE
+#endif
+#endif
+
+
+#if defined(BOOST_MSVC)
 // We deliberately mix float/double's  so turn off warnings
 #pragma warning( disable : 4244 )
 // For (new since Boost 1.40) warning in Boost.Test on putenv/posix
 #pragma warning( disable : 4996 )
 
 //#pragma warning( disable : 4305 )
-#endif // defined(_MSC_VER)
+#endif // defined(BOOST_MSVC)
 
 #include <boost/config.hpp>
+#include <boost/concept_check.hpp>
 
 
 #if defined(BOOST_INTEL_CXX_VERSION)
@@ -34,21 +58,27 @@
 
 #include <boost/foreach.hpp>
 
+#include <string_from_type.hpp>
 
 // Include some always-included-for-testing files
 #if ! defined(BOOST_GEOMETRY_NO_BOOST_TEST)
 
 // Until Boost.Test fixes it, silence warning issued by clang:
-// warning: unused variable 'check_is_close' [-Wunused-variable]
 #ifdef __clang__
 # pragma clang diagnostic push
+// warning: unused variable 'check_is_close' [-Wunused-variable]
 # pragma clang diagnostic ignored "-Wunused-variable"
+// warnings when -Wconversion is set
+# pragma clang diagnostic ignored "-Wsign-conversion"
+# pragma clang diagnostic ignored "-Wshorten-64-to-32"
 #endif
 
 # include <boost/test/floating_point_comparison.hpp>
+#ifndef BOOST_TEST_MODULE
 # include <boost/test/included/test_exec_monitor.hpp>
 //#  include <boost/test/included/prg_exec_monitor.hpp>
 # include <boost/test/impl/execution_monitor.ipp>
+#endif
 
 #ifdef __clang__
 # pragma clang diagnostic pop
@@ -83,52 +113,13 @@
 namespace bg = boost::geometry;
 
 
-template <typename T>
-struct string_from_type {};
-
-template <> struct string_from_type<void>
-{ static std::string name() { return "v"; }  };
-
-template <> struct string_from_type<float>
-{ static std::string name() { return "f"; }  };
-
-template <> struct string_from_type<double>
-{ static std::string name() { return "d"; }  };
-
-template <> struct string_from_type<long double>
-{ static std::string name() { return "e"; }  };
-
-template <> struct string_from_type<int>
-{ static std::string name() { return "i"; }  };
-
-#if defined(HAVE_TTMATH)
-    template <> struct string_from_type<ttmath_big>
-    { static std::string name() { return "t"; }  };
-#endif
-
-#if defined(BOOST_RATIONAL_HPP) 
-template <typename T> struct string_from_type<boost::rational<T> >
-{ static std::string name() { return "r"; }  };
-#endif
-
-
-#if defined(HAVE_GMP)
-    template <> struct string_from_type<boost::numeric_adaptor::gmp_value_type>
-    { static std::string name() { return "g"; }  };
-#endif
-
-#if defined(HAVE_CLN)
-    template <> struct string_from_type<boost::numeric_adaptor::cln_value_type>
-    { static std::string name() { return "c"; }  };
-#endif
-
-
 template <typename CoordinateType, typename T1, typename T2>
 inline T1 if_typed_tt(T1 value_tt, T2 value)
 {
 #if defined(HAVE_TTMATH)
     return boost::is_same<CoordinateType, ttmath_big>::type::value ? value_tt : value;
 #else
+    boost::ignore_unused_variable_warning(value_tt);
     return value;
 #endif
 }
@@ -172,7 +163,7 @@ struct mathematical_policy
     {
         return 90 - value;
     }
-    
+
 };
 
 
