@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2011. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -24,12 +24,9 @@
 #include <boost/interprocess/detail/config_begin.hpp>
 #include <boost/interprocess/sync/scoped_lock.hpp>
 #include <boost/interprocess/detail/os_thread_functions.hpp>
-#include <boost/thread/thread.hpp>
-#include <boost/thread/xtime.hpp>
 
 #include <boost/date_time/posix_time/posix_time_types.hpp>
-#include <algorithm>
-#include <iostream>
+#include <boost/version.hpp>
 
 namespace boost {
 namespace interprocess {
@@ -60,32 +57,24 @@ inline bool in_range(const boost::posix_time::ptime& xt, int secs=1)
     return (xt > min) && (max > xt);
 }
 
-boost::xtime xsecs(int secs)
-{
-   boost::xtime ret;
-   boost::xtime_get(&ret, boost::TIME_UTC_);
-   ret.sec += secs;
-   return ret;
-}
-
 template <typename P>
 class thread_adapter
 {
    public:
    thread_adapter(void (*func)(void*, P &), void* param1, P &param2)
-      : _func(func), _param1(param1) ,_param2(param2){ }
-   void operator()() const { _func(_param1, _param2); }
+      : func_(func), param1_(param1) ,param2_(param2){ }
+   void operator()() const { func_(param1_, param2_); }
 
    private:
-   void (*_func)(void*, P &);
-   void* _param1;
-   P& _param2;
+   void (*func_)(void*, P &);
+   void* param1_;
+   P& param2_;
 };
 
 template <typename P>
 struct data
 {
-   data(int id, int secs=0)
+   explicit data(int id, int secs=0)
       : m_id(id), m_value(-1), m_secs(secs), m_error(no_error)
    {}
    int            m_id;
