@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -9,12 +9,6 @@
 //////////////////////////////////////////////////////////////////////////////
 
 #include <boost/interprocess/detail/config_begin.hpp>
-#include <algorithm>
-#include <memory>
-#include <vector>
-#include <iostream>
-#include <functional>
-
 #include <boost/interprocess/managed_shared_memory.hpp>
 #include <boost/interprocess/containers/vector.hpp>
 #include <boost/interprocess/allocators/allocator.hpp>
@@ -28,10 +22,6 @@
 
 using namespace boost::interprocess;
 
-//Explicit instantiation to detect compilation errors
-template class boost::interprocess::vector<test::movable_and_copyable_int, 
-   test::dummy_test_allocator<test::movable_and_copyable_int> >;
-
 int test_expand_bwd()
 {
    //Now test all back insertion possibilities
@@ -67,29 +57,8 @@ int test_expand_bwd()
    return 0;
 }
 
-class recursive_vector
-{
-   public:
-   int id_;
-   vector<recursive_vector> vector_;
-};
-
-void recursive_vector_test()//Test for recursive types
-{
-   vector<recursive_vector> recursive_vector_vector;
-}
-
 int main()
 {
-   recursive_vector_test();
-   {
-      //Now test move semantics
-      vector<recursive_vector> original;
-      vector<recursive_vector> move_ctor(boost::interprocess::move(original));
-      vector<recursive_vector> move_assign;
-      move_assign = boost::interprocess::move(move_ctor);
-      move_assign.swap(original);
-   }
    typedef allocator<int, managed_shared_memory::segment_manager> ShmemAllocator;
    typedef vector<int, ShmemAllocator> MyVector;
 
