@@ -1,5 +1,5 @@
 /*
- *          Copyright Andrey Semashev 2007 - 2013.
+ *          Copyright Andrey Semashev 2007 - 2015.
  * Distributed under the Boost Software License, Version 1.0.
  *    (See accompanying file LICENSE_1_0.txt or copy at
  *          http://www.boost.org/LICENSE_1_0.txt)
@@ -13,6 +13,7 @@
  *         at http://www.boost.org/doc/libs/release/libs/log/doc/html/index.html.
  */
 
+#include <boost/log/detail/config.hpp>
 #include <ctime>
 #include <cctype>
 #include <cwctype>
@@ -21,7 +22,6 @@
 #include <cstdlib>
 #include <cstddef>
 #include <list>
-#include <memory>
 #include <string>
 #include <locale>
 #include <ostream>
@@ -31,6 +31,7 @@
 #include <stdexcept>
 #include <boost/ref.hpp>
 #include <boost/bind.hpp>
+#include <boost/cstdint.hpp>
 #include <boost/smart_ptr/make_shared_object.hpp>
 #include <boost/enable_shared_from_this.hpp>
 #include <boost/throw_exception.hpp>
@@ -47,17 +48,13 @@
 #include <boost/intrusive/options.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/date_time/gregorian/gregorian_types.hpp>
-#include <boost/spirit/include/qi_core.hpp>
-#include <boost/spirit/include/qi_lit.hpp>
-#include <boost/log/detail/snprintf.hpp>
+#include <boost/spirit/home/qi/numeric/numeric_utils.hpp>
 #include <boost/log/detail/singleton.hpp>
 #include <boost/log/detail/light_function.hpp>
-#include <boost/log/utility/functional/bind_assign.hpp>
-#include <boost/log/utility/functional/as_action.hpp>
 #include <boost/log/exceptions.hpp>
 #include <boost/log/attributes/time_traits.hpp>
 #include <boost/log/sinks/text_file_backend.hpp>
-#include <boost/log/sinks/text_multifile_backend.hpp>
+#include "unique_ptr.hpp"
 
 #if !defined(BOOST_LOG_NO_THREADS)
 #include <boost/thread/locks.hpp>
@@ -135,7 +132,11 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         static const char_type dot = '.';
         static const char_type newline = '\n';
 
-        static bool is_digit(char c) { return (isdigit(c) != 0); }
+        static bool is_digit(char c)
+        {
+            using namespace std;
+            return (isdigit(c) != 0);
+        }
         static std::string default_file_name_pattern() { return "%5N.log"; }
     };
 
@@ -180,7 +181,11 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         static const char_type dot = L'.';
         static const char_type newline = L'\n';
 
-        static bool is_digit(wchar_t c) { return (iswdigit(c) != 0); }
+        static bool is_digit(wchar_t c)
+        {
+            using namespace std;
+            return (iswdigit(c) != 0);
+        }
         static std::wstring default_file_name_pattern() { return L"%5N.log"; }
     };
 
@@ -213,31 +218,28 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         typedef date_time::time_facet< posix_time::ptime, path_char_type > time_facet_type;
 
     private:
-        time_facet_type* m_pFacet;
+        mutable time_facet_type m_Facet;
         mutable std::basic_ostringstream< path_char_type > m_Stream;
 
     public:
         //! Constructor
-        date_and_time_formatter() : m_pFacet(NULL)
+        date_and_time_formatter() : m_Facet(1u)
         {
-            std::auto_ptr< time_facet_type > pFacet(new time_facet_type());
-            m_pFacet = pFacet.get();
-            std::locale loc(m_Stream.getloc(), m_pFacet);
-            pFacet.release();
-            m_Stream.imbue(loc);
         }
         //! Copy constructor
-        date_and_time_formatter(date_and_time_formatter const& that) :
-            m_pFacet(that.m_pFacet)
+        date_and_time_formatter(date_and_time_formatter const& that) : m_Facet(1u)
         {
-            m_Stream.imbue(that.m_Stream.getloc());
         }
         //! The method formats the current date and time according to the format string str and writes the result into it
         path_string_type operator()(path_string_type const& pattern, unsigned int counter) const
         {
-            m_pFacet->format(pattern.c_str());
+            m_Facet.format(pattern.c_str());
             m_Stream.str(path_string_type());
-            m_Stream << boost::log::attributes::local_time_traits::get_clock();
+            // Note: the regular operator<< fails because std::use_facet fails to find the facet in the locale because
+            // the facet type in Boost.DateTime has hidden visibility. See this ticket:
+            // https://svn.boost.org/trac/boost/ticket/11707
+            std::ostreambuf_iterator< path_char_type > sbuf_it(m_Stream);
+            m_Facet.put(sbuf_it, m_Stream, m_Stream.fill(), boost::log::attributes::local_time_traits::get_clock());
             if (m_Stream.good())
             {
                 return m_Stream.str();
@@ -248,6 +250,8 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
                 return pattern;
             }
         }
+
+        BOOST_DELETED_FUNCTION(date_and_time_formatter& operator= (date_and_time_formatter const&))
     };
 
     //! The functor formats the file counter into the file name
@@ -293,6 +297,8 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
 
             return file_name;
         }
+
+        BOOST_DELETED_FUNCTION(file_counter_formatter& operator= (file_counter_formatter const&))
     };
 
     //! The function returns the pattern as the file name
@@ -319,32 +325,62 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         {
             return m_Pattern;
         }
+
+        BOOST_DELETED_FUNCTION(empty_formatter& operator= (empty_formatter const&))
     };
 
     //! The function parses the format placeholder for file counter
     bool parse_counter_placeholder(path_string_type::const_iterator& it, path_string_type::const_iterator end, unsigned int& width)
     {
+        typedef qi::extract_uint< unsigned int, 10, 1, -1 > width_extract;
         typedef file_char_traits< path_char_type > traits_t;
-        return qi::parse
-        (
-            it, end,
-            (
-                -(
-                    qi::lit(traits_t::zero) |
-                    qi::lit(traits_t::plus) |
-                    qi::lit(traits_t::minus) |
-                    qi::lit(traits_t::space)
-                ) >>
-                -(qi::uint_[boost::log::as_action(boost::log::bind_assign(width))]) >>
-                -(qi::lit(traits_t::dot) >> qi::uint_) >>
-                qi::lit(traits_t::number_placeholder)
-            )
-        );
+        if (it == end)
+            return false;
+
+        path_char_type c = *it;
+        if (c == traits_t::zero || c == traits_t::space || c == traits_t::plus || c == traits_t::minus)
+        {
+            // Skip filler and alignment specification
+            ++it;
+            if (it == end)
+                return false;
+            c = *it;
+        }
+
+        if (traits_t::is_digit(c))
+        {
+            // Parse width
+            if (!width_extract::call(it, end, width))
+                return false;
+            if (it == end)
+                return false;
+            c = *it;
+        }
+
+        if (c == traits_t::dot)
+        {
+            // Skip precision
+            ++it;
+            while (it != end && traits_t::is_digit(*it))
+                ++it;
+            if (it == end)
+                return false;
+            c = *it;
+        }
+
+        if (c == traits_t::number_placeholder)
+        {
+            ++it;
+            return true;
+        }
+
+        return false;
     }
 
     //! The function matches the file name and the pattern
     bool match_pattern(path_string_type const& file_name, path_string_type const& pattern, unsigned int& file_counter)
     {
+        typedef qi::extract_uint< unsigned int, 10, 1, -1 > file_counter_extract;
         typedef file_char_traits< path_char_type > traits_t;
 
         struct local
@@ -439,9 +475,10 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
                         path_string_type::const_iterator f = f_it;
                         if (!local::scan_digits(f, f_end, width))
                             return false;
-                        for (; f != f_end && traits_t::is_digit(*f); ++f);
+                        while (f != f_end && traits_t::is_digit(*f))
+                            ++f;
 
-                        if (!qi::parse(f_it, f, qi::uint_, file_counter))
+                        if (!file_counter_extract::call(f_it, f, file_counter))
                             return false;
 
                         p_it = p;
@@ -508,6 +545,9 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         uintmax_t m_MaxSize;
         //! Free space lower limit
         uintmax_t m_MinFreeSpace;
+        //! File count upper limit
+        uintmax_t m_MaxFiles;
+
         //! The current path at the point when the collector is created
         /*
          * The special member is required to calculate absolute paths with no
@@ -528,7 +568,8 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
             shared_ptr< file_collector_repository > const& repo,
             filesystem::path const& target_dir,
             uintmax_t max_size,
-            uintmax_t min_free_space);
+            uintmax_t min_free_space,
+            uintmax_t max_files);
 
         //! Destructor
         ~file_collector();
@@ -541,7 +582,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
             file::scan_method method, filesystem::path const& pattern, unsigned int* counter);
 
         //! The function updates storage restrictions
-        void update(uintmax_t max_size, uintmax_t min_free_space);
+        void update(uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files);
 
         //! The function checks if the directory is governed by this collector
         bool is_governed(filesystem::path const& dir) const
@@ -594,7 +635,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     public:
         //! Finds or creates a file collector
         shared_ptr< file::collector > get_collector(
-            filesystem::path const& target_dir, uintmax_t max_size, uintmax_t min_free_space);
+            filesystem::path const& target_dir, uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files);
 
         //! Removes the file collector from the list
         void remove_collector(file_collector* p);
@@ -612,11 +653,13 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         shared_ptr< file_collector_repository > const& repo,
         filesystem::path const& target_dir,
         uintmax_t max_size,
-        uintmax_t min_free_space
+        uintmax_t min_free_space,
+        uintmax_t max_files
     ) :
         m_pRepository(repo),
         m_MaxSize(max_size),
         m_MinFreeSpace(min_free_space),
+        m_MaxFiles(max_files),
         m_BasePath(filesystem::current_path()),
         m_TotalSize(0)
     {
@@ -678,7 +721,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         uintmax_t free_space = m_MinFreeSpace ? filesystem::space(m_StorageDir).available : static_cast< uintmax_t >(0);
         file_list::iterator it = m_Files.begin(), end = m_Files.end();
         while (it != end &&
-            (m_TotalSize + info.m_Size > m_MaxSize || (m_MinFreeSpace && m_MinFreeSpace > free_space)))
+            (m_TotalSize + info.m_Size > m_MaxSize || (m_MinFreeSpace && m_MinFreeSpace > free_space) || m_MaxFiles <= m_Files.size()))
         {
             file_info& old_info = *it;
             if (filesystem::exists(old_info.m_Path) && filesystem::is_regular_file(old_info.m_Path))
@@ -793,18 +836,19 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
     }
 
     //! The function updates storage restrictions
-    void file_collector::update(uintmax_t max_size, uintmax_t min_free_space)
+    void file_collector::update(uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files)
     {
         BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
 
         m_MaxSize = (std::min)(m_MaxSize, max_size);
         m_MinFreeSpace = (std::max)(m_MinFreeSpace, min_free_space);
+        m_MaxFiles = (std::min)(m_MaxFiles, max_files);
     }
 
 
     //! Finds or creates a file collector
     shared_ptr< file::collector > file_collector_repository::get_collector(
-        filesystem::path const& target_dir, uintmax_t max_size, uintmax_t min_free_space)
+        filesystem::path const& target_dir, uintmax_t max_size, uintmax_t min_free_space, uintmax_t max_files)
     {
         BOOST_LOG_EXPR_IF_MT(lock_guard< mutex > lock(m_Mutex);)
 
@@ -815,7 +859,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         {
             // This may throw if the collector is being currently destroyed
             p = it->shared_from_this();
-            p->update(max_size, min_free_space);
+            p->update(max_size, min_free_space, max_files);
         }
         catch (bad_weak_ptr&)
         {
@@ -824,7 +868,7 @@ BOOST_LOG_ANONYMOUS_NAMESPACE {
         if (!p)
         {
             p = boost::make_shared< file_collector >(
-                file_collector_repository::get(), target_dir, max_size, min_free_space);
+                file_collector_repository::get(), target_dir, max_size, min_free_space, max_files);
             m_Collectors.push_back(*p);
         }
 
@@ -871,9 +915,10 @@ namespace aux {
     BOOST_LOG_API shared_ptr< collector > make_collector(
         filesystem::path const& target_dir,
         uintmax_t max_size,
-        uintmax_t min_free_space)
+        uintmax_t min_free_space,
+        uintmax_t max_files)
     {
-        return file_collector_repository::get()->get_collector(target_dir, max_size, min_free_space);
+        return file_collector_repository::get()->get_collector(target_dir, max_size, min_free_space, max_files);
     }
 
 } // namespace aux
@@ -1130,16 +1175,36 @@ BOOST_LOG_API void text_file_backend::auto_flush(bool f)
 BOOST_LOG_API void text_file_backend::consume(record_view const& rec, string_type const& formatted_message)
 {
     typedef file_char_traits< string_type::value_type > traits_t;
-    if
+
+    bool no_new_filename = false;
+    if (!m_pImpl->m_File.good())
+    {
+        // The file stream is not operational. One possible reason is that there is no more free space
+        // on the file system. In this case it is possible that this log record will fail to be written as well,
+        // leaving the newly creted file empty. Eventually this results in lots of empty log files.
+        // We should take precautions to avoid this. https://svn.boost.org/trac/boost/ticket/11016
+        close_file();
+
+        system::error_code ec;
+        uintmax_t size = filesystem::file_size(m_pImpl->m_FileName, ec);
+        if (!!ec || size == 0)
+        {
+            // To reuse the empty file avoid re-generating the new file name later
+            no_new_filename = true;
+        }
+        else if (!!m_pImpl->m_pFileCollector)
+        {
+            // Complete file rotation
+            m_pImpl->m_pFileCollector->store_file(m_pImpl->m_FileName);
+        }
+    }
+    else if
     (
+        m_pImpl->m_File.is_open() &&
         (
-            m_pImpl->m_File.is_open() &&
-            (
-                m_pImpl->m_CharactersWritten + formatted_message.size() >= m_pImpl->m_FileRotationSize ||
-                (!m_pImpl->m_TimeBasedRotation.empty() && m_pImpl->m_TimeBasedRotation())
-            )
-        ) ||
-        !m_pImpl->m_File.good()
+            m_pImpl->m_CharactersWritten + formatted_message.size() >= m_pImpl->m_FileRotationSize ||
+            (!m_pImpl->m_TimeBasedRotation.empty() && m_pImpl->m_TimeBasedRotation())
+        )
     )
     {
         rotate_file();
@@ -1147,17 +1212,18 @@ BOOST_LOG_API void text_file_backend::consume(record_view const& rec, string_typ
 
     if (!m_pImpl->m_File.is_open())
     {
-        m_pImpl->m_FileName = m_pImpl->m_StorageDir / m_pImpl->m_FileNameGenerator(m_pImpl->m_FileCounter++);
+        if (!no_new_filename)
+            m_pImpl->m_FileName = m_pImpl->m_StorageDir / m_pImpl->m_FileNameGenerator(m_pImpl->m_FileCounter++);
 
         filesystem::create_directories(m_pImpl->m_FileName.parent_path());
+
         m_pImpl->m_File.open(m_pImpl->m_FileName, m_pImpl->m_FileOpenMode);
         if (!m_pImpl->m_File.is_open())
         {
-            filesystem_error err(
+            BOOST_THROW_EXCEPTION(filesystem_error(
                 "Failed to open file for writing",
                 m_pImpl->m_FileName,
-                system::error_code(system::errc::io_error, system::generic_category()));
-            BOOST_THROW_EXCEPTION(err);
+                system::error_code(system::errc::io_error, system::generic_category())));
         }
 
         if (!m_pImpl->m_OpenHandler.empty())
@@ -1222,50 +1288,76 @@ BOOST_LOG_API void text_file_backend::set_file_name_pattern_internal(filesystem:
 
         ++placeholder_count;
 
-        if (!counter_found && parse_counter_placeholder(it, end, width))
+        if (!counter_found)
         {
-            // We've found the file counter placeholder in the pattern
-            counter_found = true;
-            counter_pos = placeholder_begin - name_pattern.begin();
-            name_pattern.erase(counter_pos, it - placeholder_begin);
-            --placeholder_count;
-            it = name_pattern.begin() + counter_pos;
-            end = name_pattern.end();
+            path_string_type::const_iterator it2 = it;
+            if (parse_counter_placeholder(it2, end, width))
+            {
+                // We've found the file counter placeholder in the pattern
+                counter_found = true;
+                counter_pos = placeholder_begin - name_pattern.begin();
+                name_pattern.erase(counter_pos, it2 - placeholder_begin);
+                --placeholder_count;
+                it = name_pattern.begin() + counter_pos;
+                end = name_pattern.end();
+            }
         }
     }
     while (it != end);
 
     // Construct the formatter functor
-    unsigned int choice = (static_cast< unsigned int >(placeholder_count > 0) << 1) |
-                          static_cast< unsigned int >(counter_found);
-    switch (choice)
+    if (placeholder_count > 0)
     {
-    case 1: // Only counter placeholder in the pattern
+        if (counter_found)
+        {
+            // Both counter and date/time placeholder in the pattern
+            m_pImpl->m_FileNameGenerator = boost::bind(date_and_time_formatter(),
+                boost::bind(file_counter_formatter(counter_pos, width), name_pattern, _1), _1);
+        }
+        else
+        {
+            // Only date/time placeholders in the pattern
+            m_pImpl->m_FileNameGenerator =
+                boost::bind(date_and_time_formatter(), name_pattern, _1);
+        }
+    }
+    else if (counter_found)
+    {
+        // Only counter placeholder in the pattern
         m_pImpl->m_FileNameGenerator =
             boost::bind(file_counter_formatter(counter_pos, width), name_pattern, _1);
-        break;
-    case 2: // Only date/time placeholders in the pattern
-        m_pImpl->m_FileNameGenerator =
-            boost::bind(date_and_time_formatter(), name_pattern, _1);
-        break;
-    case 3: // Counter and date/time placeholder in the pattern
-        m_pImpl->m_FileNameGenerator = boost::bind(date_and_time_formatter(),
-            boost::bind(file_counter_formatter(counter_pos, width), name_pattern, _1), _1);
-        break;
-    default: // No placeholders detected
-        m_pImpl->m_FileNameGenerator = empty_formatter(name_pattern);
-        break;
     }
+    else
+    {
+        // No placeholders detected
+        m_pImpl->m_FileNameGenerator = empty_formatter(name_pattern);
+    }
+}
+
+//! Closes the currently open file
+void text_file_backend::close_file()
+{
+    if (!m_pImpl->m_CloseHandler.empty())
+    {
+        // Rationale: We should call the close handler even if the stream is !good() because
+        // writing the footer may not be the only thing the handler does. However, there is
+        // a chance that the file had become writable since the last failure (e.g. there was
+        // no space left to write the last record, but it got freed since then), so if the handler
+        // attempts to write a footer it may succeed now. For this reason we clear the stream state
+        // and let the handler have a try.
+        m_pImpl->m_File.clear();
+        m_pImpl->m_CloseHandler(m_pImpl->m_File);
+    }
+    m_pImpl->m_File.close();
+    m_pImpl->m_File.clear();
+    m_pImpl->m_CharactersWritten = 0;
 }
 
 //! The method rotates the file
 BOOST_LOG_API void text_file_backend::rotate_file()
 {
-    if (!m_pImpl->m_CloseHandler.empty())
-        m_pImpl->m_CloseHandler(m_pImpl->m_File);
-    m_pImpl->m_File.close();
-    m_pImpl->m_File.clear();
-    m_pImpl->m_CharactersWritten = 0;
+    close_file();
+
     if (!!m_pImpl->m_pFileCollector)
         m_pImpl->m_pFileCollector->store_file(m_pImpl->m_FileName);
 }
@@ -1312,67 +1404,6 @@ BOOST_LOG_API uintmax_t text_file_backend::scan_for_files(file::scan_method meth
     }
 }
 
-
-////////////////////////////////////////////////////////////////////////////////
-//  Multifile sink backend implementation
-////////////////////////////////////////////////////////////////////////////////
-//! Sink implementation data
-struct text_multifile_backend::implementation
-{
-    //! File name composer
-    file_name_composer_type m_FileNameComposer;
-    //! Base path for absolute path composition
-    const filesystem::path m_BasePath;
-    //! File stream
-    filesystem::ofstream m_File;
-
-    implementation() :
-        m_BasePath(filesystem::current_path())
-    {
-    }
-
-    //! Makes relative path absolute with respect to the base path
-    filesystem::path make_absolute(filesystem::path const& p)
-    {
-        return filesystem::absolute(p, m_BasePath);
-    }
-};
-
-//! Default constructor
-BOOST_LOG_API text_multifile_backend::text_multifile_backend() : m_pImpl(new implementation())
-{
-}
-
-//! Destructor
-BOOST_LOG_API text_multifile_backend::~text_multifile_backend()
-{
-    delete m_pImpl;
-}
-
-//! The method sets the file name composer
-BOOST_LOG_API void text_multifile_backend::set_file_name_composer_internal(file_name_composer_type const& composer)
-{
-    m_pImpl->m_FileNameComposer = composer;
-}
-
-//! The method writes the message to the sink
-BOOST_LOG_API void text_multifile_backend::consume(record_view const& rec, string_type const& formatted_message)
-{
-    typedef file_char_traits< string_type::value_type > traits_t;
-    if (!m_pImpl->m_FileNameComposer.empty())
-    {
-        filesystem::path file_name = m_pImpl->make_absolute(m_pImpl->m_FileNameComposer(rec));
-        filesystem::create_directories(file_name.parent_path());
-        m_pImpl->m_File.open(file_name, std::ios_base::out | std::ios_base::app);
-        if (m_pImpl->m_File.is_open())
-        {
-            m_pImpl->m_File.write(formatted_message.data(), static_cast< std::streamsize >(formatted_message.size()));
-            m_pImpl->m_File.put(traits_t::newline);
-            m_pImpl->m_File.close();
-        }
-    }
-}
-
 } // namespace sinks
 
 BOOST_LOG_CLOSE_NAMESPACE // namespace log
