@@ -39,8 +39,8 @@ class back_insert_iterator
 {
 private:
    Seq* container;
-   typedef const typename Seq::value_type value_type;
 public:
+   typedef const typename Seq::value_type value_type;
    typedef Seq                  container_type;
    typedef std::output_iterator_tag  iterator_category;
 
@@ -73,7 +73,7 @@ void compare_result(const MR1& w1, const MR2& w2, boost::mpl::int_<2> const*)
    typedef typename MR2::value_type MR2_value_type;
    typedef typename MR2_value_type::const_iterator MR2_iterator_type;
    typedef boost::u16_to_u32_iterator<MR2_iterator_type> iterator_type;
-   typedef typename MR1::size_type size_type;
+   //typedef typename MR1::size_type size_type;
    if(w1.size() != w2.size())
    {
       BOOST_REGEX_TEST_ERROR("Size mismatch in match_results class", UChar32);
@@ -86,7 +86,7 @@ void compare_result(const MR1& w1, const MR2& w2, boost::mpl::int_<2> const*)
          {
             BOOST_REGEX_TEST_ERROR("Matched mismatch in match_results class", UChar32);
          }
-         if((w1.position(i) != boost::re_detail::distance(iterator_type(w2.prefix().first), iterator_type(w2[i].first))) || (w1.length(i) != boost::re_detail::distance(iterator_type(w2[i].first), iterator_type(w2[i].second))))
+         if((w1.position(i) != boost::BOOST_REGEX_DETAIL_NS::distance(iterator_type(w2.prefix().first), iterator_type(w2[i].first))) || (w1.length(i) != boost::BOOST_REGEX_DETAIL_NS::distance(iterator_type(w2[i].first), iterator_type(w2[i].second))))
          {
             BOOST_REGEX_TEST_ERROR("Iterator mismatch in match_results class", UChar32);
          }
@@ -103,7 +103,7 @@ void compare_result(const MR1& w1, const MR2& w2, boost::mpl::int_<1> const*)
    typedef typename MR2::value_type MR2_value_type;
    typedef typename MR2_value_type::const_iterator MR2_iterator_type;
    typedef boost::u8_to_u32_iterator<MR2_iterator_type> iterator_type;
-   typedef typename MR1::size_type size_type;
+   //typedef typename MR1::size_type size_type;
    if(w1.size() != w2.size())
    {
       BOOST_REGEX_TEST_ERROR("Size mismatch in match_results class", UChar32);
@@ -116,7 +116,7 @@ void compare_result(const MR1& w1, const MR2& w2, boost::mpl::int_<1> const*)
          {
             BOOST_REGEX_TEST_ERROR("Matched mismatch in match_results class", UChar32);
          }
-         if((w1.position(i) != boost::re_detail::distance(iterator_type(w2.prefix().first), iterator_type(w2[i].first))) || (w1.length(i) != boost::re_detail::distance(iterator_type(w2[i].first), iterator_type(w2[i].second))))
+         if((w1.position(i) != boost::BOOST_REGEX_DETAIL_NS::distance(iterator_type(w2.prefix().first), iterator_type(w2[i].first))) || (w1.length(i) != boost::BOOST_REGEX_DETAIL_NS::distance(iterator_type(w2[i].first), iterator_type(w2[i].second))))
          {
             BOOST_REGEX_TEST_ERROR("Iterator mismatch in match_results class", UChar32);
          }
@@ -205,7 +205,10 @@ void test_icu(const wchar_t&, const test_regex_search_tag& )
    std::copy(test_info<wchar_t>::expression().begin(), test_info<wchar_t>::expression().end(), std::back_inserter(expression));
 #endif
    boost::regex_constants::syntax_option_type syntax_options = test_info<UChar32>::syntax_options();
-   try{
+#ifndef BOOST_NO_EXCEPTIONS
+   try
+#endif
+   {
 #if !defined(BOOST_NO_MEMBER_TEMPLATES) && !defined(__IBMCPP__)
       r.assign(expression.begin(), expression.end(), syntax_options);
 #else
@@ -338,27 +341,29 @@ void test_icu(const wchar_t&, const test_regex_search_tag& )
       //
       test_icu_grep(r, search_text);
    }
+#ifndef BOOST_NO_EXCEPTIONS
    catch(const boost::bad_expression& e)
    {
       BOOST_REGEX_TEST_ERROR("Expression did not compile when it should have done: " << e.what(), UChar32);
    }
-   catch(const std::runtime_error& r)
+   catch(const std::runtime_error& e)
    {
-      BOOST_REGEX_TEST_ERROR("Received an unexpected std::runtime_error: " << r.what(), UChar32);
+      BOOST_REGEX_TEST_ERROR("Received an unexpected std::runtime_error: " << e.what(), UChar32);
    }
-   catch(const std::exception& r)
+   catch(const std::exception& e)
    {
-      BOOST_REGEX_TEST_ERROR("Received an unexpected std::exception: " << r.what(), UChar32);
+      BOOST_REGEX_TEST_ERROR("Received an unexpected std::exception: " << e.what(), UChar32);
    }
    catch(...)
    {
       BOOST_REGEX_TEST_ERROR("Received an unexpected exception of unknown type", UChar32);
    }
+#endif
 }
 
 void test_icu(const wchar_t&, const test_invalid_regex_tag&)
 {
-   typedef boost::u16_to_u32_iterator<std::wstring::const_iterator, ::UChar32> conv_iterator;
+   //typedef boost::u16_to_u32_iterator<std::wstring::const_iterator, ::UChar32> conv_iterator;
    std::vector< ::UChar32> expression;
 #ifndef BOOST_NO_TEMPLATED_ITERATOR_CONSTRUCTORS
    expression.assign(test_info<wchar_t>::expression().begin(), test_info<wchar_t>::expression().end());
@@ -377,7 +382,9 @@ void test_icu(const wchar_t&, const test_invalid_regex_tag&)
    //
    // try it with exceptions disabled first:
    //
+#ifndef BOOST_NO_EXCEPTIONS
    try
+#endif
    {
 #if !defined(BOOST_NO_MEMBER_TEMPLATES) && !defined(__IBMCPP__)
       if(0 == r.assign(expression.begin(), expression.end(), syntax_options | boost::regex_constants::no_except).status())
@@ -392,15 +399,20 @@ void test_icu(const wchar_t&, const test_invalid_regex_tag&)
          BOOST_REGEX_TEST_ERROR("Expression compiled when it should not have done so.", wchar_t);
       }
    }
+#ifndef BOOST_NO_EXCEPTIONS
    catch(...)
    {
       BOOST_REGEX_TEST_ERROR("Unexpected exception thrown.", wchar_t);
    }
+#endif
    //
    // now try again with exceptions:
    //
    bool have_catch = false;
-   try{
+#ifndef BOOST_NO_EXCEPTIONS
+   try
+#endif
+   {
 #if !defined(BOOST_NO_MEMBER_TEMPLATES) && !defined(__IBMCPP__)
       r.assign(expression.begin(), expression.end(), syntax_options);
 #else
@@ -414,25 +426,27 @@ void test_icu(const wchar_t&, const test_invalid_regex_tag&)
          have_catch = true;
 #endif
    }
+#ifndef BOOST_NO_EXCEPTIONS
    catch(const boost::bad_expression&)
    {
       have_catch = true;
    }
-   catch(const std::runtime_error& r)
+   catch(const std::runtime_error& e)
    {
       have_catch = true;
-      BOOST_REGEX_TEST_ERROR("Expected a bad_expression exception, but a std::runtime_error instead: " << r.what(), wchar_t);
+      BOOST_REGEX_TEST_ERROR("Expected a bad_expression exception, but a std::runtime_error instead: " << e.what(), wchar_t);
    }
-   catch(const std::exception& r)
+   catch(const std::exception& e)
    {
       have_catch = true;
-      BOOST_REGEX_TEST_ERROR("Expected a bad_expression exception, but a std::exception instead: " << r.what(), wchar_t);
+      BOOST_REGEX_TEST_ERROR("Expected a bad_expression exception, but a std::exception instead: " << e.what(), wchar_t);
    }
    catch(...)
    {
       have_catch = true;
       BOOST_REGEX_TEST_ERROR("Expected a bad_expression exception, but got an exception of unknown type instead", wchar_t);
    }
+#endif
    if(!have_catch)
    {
       // oops expected exception was not thrown:
@@ -498,7 +512,10 @@ void test_icu(const wchar_t&, const test_regex_replace_tag&)
 #endif
    boost::regex_constants::syntax_option_type syntax_options = test_info<UChar32>::syntax_options();
    boost::u32regex r;
-   try{
+#ifndef BOOST_NO_EXCEPTIONS
+   try
+#endif
+   {
 #if !defined(BOOST_NO_MEMBER_TEMPLATES) && !defined(__IBMCPP__)
       r.assign(expression.begin(), expression.end(), syntax_options);
 #else
@@ -564,7 +581,7 @@ void test_icu(const wchar_t&, const test_regex_replace_tag&)
          //
          // Now with UnicodeString:
          //
-         UnicodeString expression16u, text16u, format16u, result16u, found16u;
+         U_NAMESPACE_QUALIFIER UnicodeString expression16u, text16u, format16u, result16u, found16u;
          if(expression16.size())
             expression16u.setTo(&*expression16.begin(), expression16.size());
          if(text16.size())
@@ -620,22 +637,24 @@ void test_icu(const wchar_t&, const test_regex_replace_tag&)
          }
       }
    }
+#ifndef BOOST_NO_EXCEPTIONS
    catch(const boost::bad_expression& e)
    {
       BOOST_REGEX_TEST_ERROR("Expression did not compile when it should have done: " << e.what(), UChar32);
    }
-   catch(const std::runtime_error& r)
+   catch(const std::runtime_error& e)
    {
-      BOOST_REGEX_TEST_ERROR("Received an unexpected std::runtime_error: " << r.what(), UChar32);
+      BOOST_REGEX_TEST_ERROR("Received an unexpected std::runtime_error: " << e.what(), UChar32);
    }
-   catch(const std::exception& r)
+   catch(const std::exception& e)
    {
-      BOOST_REGEX_TEST_ERROR("Received an unexpected std::exception: " << r.what(), UChar32);
+      BOOST_REGEX_TEST_ERROR("Received an unexpected std::exception: " << e.what(), UChar32);
    }
    catch(...)
    {
       BOOST_REGEX_TEST_ERROR("Received an unexpected exception of unknown type", UChar32);
    }
+#endif
 }
 
 #else
@@ -647,3 +666,4 @@ void test_icu(const wchar_t&, const test_invalid_regex_tag&){}
 void test_icu(const wchar_t&, const test_regex_replace_tag&){}
 
 #endif
+
