@@ -2,7 +2,7 @@
 #define BOOST_SERIALIZATION_TEST_BASE_HPP
 
 // MS compatible compilers support #pragma once
-#if defined(_MSC_VER) && (_MSC_VER >= 1020)
+#if defined(_MSC_VER)
 # pragma once
 #endif
 
@@ -16,21 +16,21 @@
 
 //  See http://www.boost.org for updates, documentation, and revision history.
 
+#include <boost/config.hpp>
+
 #include <boost/serialization/access.hpp>
 #include <boost/serialization/assume_abstract.hpp>
 #include <boost/preprocessor/facilities/empty.hpp>
 
-#include "test_decl.hpp"
-
 #if defined(BASE_IMPORT)
-    #define DLL_DECL IMPORT_DECL
+    #define DLL_DECL BOOST_SYMBOL_IMPORT
 #elif defined(BASE_EXPORT)
-    #define DLL_DECL EXPORT_DECL
+    #define DLL_DECL BOOST_SYMBOL_EXPORT
 #else
-    #define DLL_DECL(x)
+    #define DLL_DECL
 #endif
 
-class DLL_DECL(BOOST_PP_EMPTY()) base
+class DLL_DECL base
 {
     friend class boost::serialization::access;
     template<class Archive>
