@@ -20,12 +20,12 @@
 #include <boost/archive/text_iarchive.hpp>
 
 template
-EXPORT_DECL(void) A::serialize(
+BOOST_SYMBOL_EXPORT void A::serialize(
     boost::archive::text_oarchive &ar,
     const unsigned int /* file_version */
 );
 template
-EXPORT_DECL(void) A::serialize(
+BOOST_SYMBOL_EXPORT void A::serialize(
     boost::archive::text_iarchive &ar,
     const unsigned int /* file_version */
 );
@@ -36,12 +36,12 @@ EXPORT_DECL(void) A::serialize(
 #include <boost/archive/polymorphic_iarchive.hpp>
 
 template
-EXPORT_DECL(void) A::serialize(
+BOOST_SYMBOL_EXPORT void A::serialize(
     boost::archive::polymorphic_oarchive &ar,
     const unsigned int /* file_version */
 );
 template
-EXPORT_DECL(void) A::serialize(
+BOOST_SYMBOL_EXPORT void A::serialize(
     boost::archive::polymorphic_iarchive &ar,
     const unsigned int /* file_version */
 );
