@@ -19,7 +19,7 @@
 // upgrade_lock(mutex_type& m, adopt_lock_t);
 
 
-#include <boost/thread/locks.hpp>
+#include <boost/thread/lock_types.hpp>
 #include <boost/thread/shared_mutex.hpp>
 #include <boost/detail/lightweight_test.hpp>
 
