@@ -13,28 +13,11 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests #ifdef, #ifndef directives.
+// Tests error reporting: #define syntax errors.
 
-#define MACRO_0     0
-#define MACRO_1     1
-
-// 15.1:  #ifdef directive.
-//R #line 25 "t_5_020.cpp"
-//R true
-#ifdef MACRO_1     /* Valid block  */
-true
-#else
-false
-#endif
-
-// 15.2: #ifndef directive.
-//R #line 36 "t_5_020.cpp"
-//R true
-#ifndef MACRO_1
-false
-#else               /* Valid block  */
-true
-#endif
+// 18.6: Empty parameter list.
+//E t_6_035.cpp(20): error: ill formed preprocessor directive: #define math( op, a, )      op( (a), (b))
+#define math( op, a, )      op( (a), (b))
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
