@@ -16,11 +16,11 @@
 #include "quickbook.hpp"
 #include "utils.hpp"
 #include "files.hpp"
-#include "input_path.hpp"
+#include "native_text.hpp"
 #include "state.hpp"
-#include "actions.hpp"
 #include "doc_info_tags.hpp"
-#include "id_manager.hpp"
+#include "document_state.hpp"
+#include "include_paths.hpp"
 
 namespace quickbook
 {
@@ -239,20 +239,20 @@ namespace quickbook
 
         if (!compatibility_version) {
             compatibility_version = use_doc_info ?
-                qbk_version_n : state.ids.compatibility_version();
+                qbk_version_n : state.document.compatibility_version();
         }
 
         // Start file, finish here if not generating document info.
 
         if (!use_doc_info)
         {
-            state.ids.start_file(compatibility_version, include_doc_id_, id_,
+            state.document.start_file(compatibility_version, include_doc_id_, id_,
                     doc_title);
             return "";
         }
 
         std::string id_placeholder =
-            state.ids.start_file_with_docinfo(
+            state.document.start_file_with_docinfo(
                 compatibility_version, include_doc_id_, id_, doc_title);
 
         // Make sure we really did have a document info block.
@@ -265,22 +265,27 @@ namespace quickbook
 
         if (!xmlbase.empty())
         {
-            xinclude_path x = calculate_xinclude_path(xmlbase, state);
+            path_parameter x = check_xinclude_path(xmlbase, state);
 
-            if (!fs::is_directory(x.path))
+            if (x.type == path_parameter::path)
             {
-                detail::outerr(xmlbase.get_file(), xmlbase.get_position())
-                    << "xmlbase \""
-                    << xmlbase.get_quickbook()
-                    << "\" isn't a directory."
-                    << std::endl;
+                quickbook_path path = resolve_xinclude_path(x.value, state);
 
-                ++state.error_count;
-            }
-            else
-            {
-                xmlbase_value = x.uri;
-                state.xinclude_base = x.path;
+                if (!fs::is_directory(path.file_path))
+                {
+                    detail::outerr(xmlbase.get_file(), xmlbase.get_position())
+                        << "xmlbase \""
+                        << xmlbase.get_quickbook()
+                        << "\" isn't a directory."
+                        << std::endl;
+
+                    ++state.error_count;
+                }
+                else
+                {
+                    xmlbase_value = dir_path_to_url(path.abstract_file_path);
+                    state.xinclude_base = path.file_path;
+                }
             }
         }
 
@@ -463,7 +468,7 @@ namespace quickbook
         if (!license.empty())
         {
             tmp << "    <legalnotice id=\""
-                << state.ids.add_id("legal", id_category::generated)
+                << state.document.add_id("legal", id_category::generated)
                 << "\">\n"
                 << "      <para>\n"
                 << "        " << doc_info_output(license, 103) << "\n"
@@ -544,18 +549,18 @@ namespace quickbook
         // *after* everything else.
 
         // Close any open sections.
-        if (!doc_type.empty() && state.ids.section_level() > 1) {
+        if (!doc_type.empty() && state.document.section_level() > 1) {
             detail::outwarn(state.current_file->path)
                 << "Missing [endsect] detected at end of file."
                 << std::endl;
 
-            while(state.ids.section_level() > 1) {
+            while(state.document.section_level() > 1) {
                 state.out << "</section>";
-                state.ids.end_section();
+                state.document.end_section();
             }
         }
 
-        state.ids.end_file();
+        state.document.end_file();
         if (!doc_type.empty()) state.out << "\n</" << doc_type << ">\n\n";
     }
 
